

class Plugin():

    def __init__(self, plugin_id):
        self.plugin_id = plugin_id

    def add_log(self, log):
        pass
        
    def add_variables(self, variables):
        pass

    def add_eventhandler(self, event_handler):
        pass

    def add_auxiliary(self, auxiliary):
        pass
        
    def apply(self):
        print(self.plugin_id)



import uuid

MAGICS = {
    'version': (int, 1, int, [1]),
    'type': (str, 'generic', str, ['object', 'system', 'generic']),
    'priority': (int, 0, int, None),
    'tags': (list, [], set, None),
    'untags': (list, [], set, None),
    'variables': (dict, {}, dict, None),
    'services': (dict, {}, dict, None),
}

SERVICE_MAGICS = {
    'plugin_id': (str, '', str, None),
    'tags': (list, [], set, None),
    'variables': (dict, {}, dict, None),
}

INTERNALS = {
    '__uuid__': lambda d: uuid.uuid4(),
}


def parse_template(d: dict = {}, magics: dict = {}, internals: dict = {}) -> dict:
    """
    Ensure incoming dict 
    """
    magics_ = magics if magics else MAGICS

    for mkey, value in magics_.items():

        type_, default_, converterf_, allowed_ = value

        value_from_template = d.get(mkey)

        if value_from_template and not isinstance(value_from_template, type_):
            raise ValueError

        current_ = value_from_template or default_

        if not isinstance(current_, converterf_):                    
            current_ = converterf_(current_)

        if allowed_ and current_ not in allowed_:
            raise ValueError

        d[mkey] = current_

    internals_ = internals if internals else INTERNALS

    for mkey, func in internals_.items():
        d[mkey] = func(d)

    return d
        


        
    


    



from pathlib import Path
import importlib
import glob
import pysquid.plugin


class PluginCollector():

    def __init__(self, paths: list = [Path('plugins')]):

        self.paths = set()
        self.pyfiles = set()
        self.pymodules = set()
        self.get_paths(paths)
        self.get_py_files()

    def get_paths(self, paths):
        
        for path in paths:

            if not isinstance(path, Path):
                continue

            if not path.exists():
                continue

            if not path.is_dir():
                continue

            self.paths.add(path)

    def get_py_files(self):

        all_files = set()

        for path in self.paths:
            files = set(glob.glob(str(path) + '/**/*.py', recursive=True))
            all_files = all_files.union(files)

        self.pyfiles = all_files

        pymodules = [f.replace('/', '.')[:-3] for f in self.pyfiles]
        self.pymodules = [f.strip('.__init__') for f in pymodules]

    def collect(self):

        plugins = {}

        for module in self.pymodules:

            imported_module = importlib.import_module(module)

            for export in imported_module.EXPORTS:

                ti = export()

                if isinstance(ti, pysquid.plugin.Plugin):
                    plugins[ti.plugin_id] = export

        return plugins
        
        
            
